"""Numba extension support for datetime.timedelta objects and their arrays.
"""
import datetime
import operator
from collections import namedtuple
import llvmlite.binding as ll
import numba
import numpy as np
import pandas as pd
from llvmlite import ir as lir
from numba.core import cgutils, types
from numba.core.imputils import lower_constant
from numba.extending import NativeValue, box, intrinsic, make_attribute_wrapper, models, overload, overload_attribute, overload_method, register_jitable, register_model, typeof_impl, unbox
from numba.parfors.array_analysis import ArrayAnalysis
import bodo
from bodo.hiframes.datetime_datetime_ext import datetime_datetime_type
from bodo.libs import hdatetime_ext
from bodo.utils.indexing import get_new_null_mask_bool_index, get_new_null_mask_int_index, get_new_null_mask_slice_index, setitem_slice_index_null_bits
from bodo.utils.typing import BodoError, get_overload_const_str, is_iterable_type, is_list_like_index_type, is_overload_constant_str
ll.add_symbol('box_datetime_timedelta_array', hdatetime_ext.
    box_datetime_timedelta_array)
ll.add_symbol('unbox_datetime_timedelta_array', hdatetime_ext.
    unbox_datetime_timedelta_array)


class NoInput:
    pass


_no_input = NoInput()


class NoInputType(types.Type):

    def __init__(self):
        super(NoInputType, self).__init__(name='NoInput')


register_model(NoInputType)(models.OpaqueModel)


@typeof_impl.register(NoInput)
def _typ_no_input(val, c):
    return NoInputType()


@lower_constant(NoInputType)
def constant_no_input(context, builder, ty, pyval):
    return context.get_dummy_value()


class PDTimeDeltaType(types.Type):

    def __init__(self):
        super(PDTimeDeltaType, self).__init__(name='PDTimeDeltaType()')


pd_timedelta_type = PDTimeDeltaType()
types.pd_timedelta_type = pd_timedelta_type


@typeof_impl.register(pd.Timedelta)
def typeof_pd_timedelta(val, c):
    return pd_timedelta_type


@register_model(PDTimeDeltaType)
class PDTimeDeltaModel(models.StructModel):

    def __init__(self, dmm, fe_type):
        fygwz__yaxmg = [('value', types.int64)]
        super(PDTimeDeltaModel, self).__init__(dmm, fe_type, fygwz__yaxmg)


@box(PDTimeDeltaType)
def box_pd_timedelta(typ, val, c):
    qolvn__gslyn = cgutils.create_struct_proxy(typ)(c.context, c.builder,
        value=val)
    hguxj__tcv = c.pyapi.long_from_longlong(qolvn__gslyn.value)
    xwg__ipx = c.pyapi.unserialize(c.pyapi.serialize_object(pd.Timedelta))
    res = c.pyapi.call_function_objargs(xwg__ipx, (hguxj__tcv,))
    c.pyapi.decref(hguxj__tcv)
    c.pyapi.decref(xwg__ipx)
    return res


@unbox(PDTimeDeltaType)
def unbox_pd_timedelta(typ, val, c):
    hguxj__tcv = c.pyapi.object_getattr_string(val, 'value')
    krpn__gtkxe = c.pyapi.long_as_longlong(hguxj__tcv)
    qolvn__gslyn = cgutils.create_struct_proxy(typ)(c.context, c.builder)
    qolvn__gslyn.value = krpn__gtkxe
    c.pyapi.decref(hguxj__tcv)
    dsw__fzzq = cgutils.is_not_null(c.builder, c.pyapi.err_occurred())
    return NativeValue(qolvn__gslyn._getvalue(), is_error=dsw__fzzq)


@lower_constant(PDTimeDeltaType)
def lower_constant_pd_timedelta(context, builder, ty, pyval):
    value = context.get_constant(types.int64, pyval.value)
    return lir.Constant.literal_struct([value])


@overload(pd.Timedelta, no_unliteral=True)
def pd_timedelta(value=_no_input, unit='ns', days=0, seconds=0,
    microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0):
    if value == _no_input:

        def impl_timedelta_kw(value=_no_input, unit='ns', days=0, seconds=0,
            microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0):
            days += weeks * 7
            hours += days * 24
            minutes += 60 * hours
            seconds += 60 * minutes
            milliseconds += 1000 * seconds
            microseconds += 1000 * milliseconds
            afhqc__klten = 1000 * microseconds
            return init_pd_timedelta(afhqc__klten)
        return impl_timedelta_kw
    if value == bodo.string_type or is_overload_constant_str(value):

        def impl_str(value=_no_input, unit='ns', days=0, seconds=0,
            microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0):
            with numba.objmode(res='pd_timedelta_type'):
                res = pd.Timedelta(value)
            return res
        return impl_str
    if value == pd_timedelta_type:
        return (lambda value=_no_input, unit='ns', days=0, seconds=0,
            microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0: value)
    if value == datetime_timedelta_type:

        def impl_timedelta_datetime(value=_no_input, unit='ns', days=0,
            seconds=0, microseconds=0, milliseconds=0, minutes=0, hours=0,
            weeks=0):
            days = value.days
            seconds = 60 * 60 * 24 * days + value.seconds
            microseconds = 1000 * 1000 * seconds + value.microseconds
            afhqc__klten = 1000 * microseconds
            return init_pd_timedelta(afhqc__klten)
        return impl_timedelta_datetime
    if not is_overload_constant_str(unit):
        raise BodoError('pd.to_timedelta(): unit should be a constant string')
    unit = pd._libs.tslibs.timedeltas.parse_timedelta_unit(
        get_overload_const_str(unit))
    jwudg__zwlo, jcz__gqgh = pd._libs.tslibs.conversion.precision_from_unit(
        unit)

    def impl_timedelta(value=_no_input, unit='ns', days=0, seconds=0,
        microseconds=0, milliseconds=0, minutes=0, hours=0, weeks=0):
        return init_pd_timedelta(value * jwudg__zwlo)
    return impl_timedelta


@intrinsic
def init_pd_timedelta(typingctx, value):

    def codegen(context, builder, signature, args):
        typ = signature.return_type
        timedelta = cgutils.create_struct_proxy(typ)(context, builder)
        timedelta.value = args[0]
        return timedelta._getvalue()
    return PDTimeDeltaType()(value), codegen


make_attribute_wrapper(PDTimeDeltaType, 'value', '_value')


@overload_attribute(PDTimeDeltaType, 'value')
@overload_attribute(PDTimeDeltaType, 'delta')
def pd_timedelta_get_value(td):

    def impl(td):
        return td._value
    return impl


@overload_attribute(PDTimeDeltaType, 'days')
def pd_timedelta_get_days(td):

    def impl(td):
        return td._value // (1000 * 1000 * 1000 * 60 * 60 * 24)
    return impl


@overload_attribute(PDTimeDeltaType, 'seconds')
def pd_timedelta_get_seconds(td):

    def impl(td):
        return td._value // (1000 * 1000 * 1000) % (60 * 60 * 24)
    return impl


@overload_attribute(PDTimeDeltaType, 'microseconds')
def pd_timedelta_get_microseconds(td):

    def impl(td):
        return td._value // 1000 % 1000000
    return impl


@overload_attribute(PDTimeDeltaType, 'nanoseconds')
def pd_timedelta_get_nanoseconds(td):

    def impl(td):
        return td._value % 1000
    return impl


@register_jitable
def _to_hours_pd_td(td):
    return td._value // (1000 * 1000 * 1000 * 60 * 60) % 24


@register_jitable
def _to_minutes_pd_td(td):
    return td._value // (1000 * 1000 * 1000 * 60) % 60


@register_jitable
def _to_seconds_pd_td(td):
    return td._value // (1000 * 1000 * 1000) % 60


@register_jitable
def _to_milliseconds_pd_td(td):
    return td._value // (1000 * 1000) % 1000


@register_jitable
def _to_microseconds_pd_td(td):
    return td._value // 1000 % 1000


Components = namedtuple('Components', ['days', 'hours', 'minutes',
    'seconds', 'milliseconds', 'microseconds', 'nanoseconds'], defaults=[0,
    0, 0, 0, 0, 0, 0])


@overload_attribute(PDTimeDeltaType, 'components', no_unliteral=True)
def pd_timedelta_get_components(td):

    def impl(td):
        a = Components(td.days, _to_hours_pd_td(td), _to_minutes_pd_td(td),
            _to_seconds_pd_td(td), _to_milliseconds_pd_td(td),
            _to_microseconds_pd_td(td), td.nanoseconds)
        return a
    return impl


@overload_method(PDTimeDeltaType, '__hash__', no_unliteral=True)
def pd_td___hash__(td):

    def impl(td):
        return hash(td._value)
    return impl


@overload_method(PDTimeDeltaType, 'to_numpy', no_unliteral=True)
@overload_method(PDTimeDeltaType, 'to_timedelta64', no_unliteral=True)
def pd_td_to_numpy(td):
    from bodo.hiframes.pd_timestamp_ext import integer_to_timedelta64

    def impl(td):
        return integer_to_timedelta64(td.value)
    return impl


@overload_method(PDTimeDeltaType, 'to_pytimedelta', no_unliteral=True)
def pd_td_to_pytimedelta(td):

    def impl(td):
        return datetime.timedelta(microseconds=np.int64(td._value / 1000))
    return impl


@overload_method(PDTimeDeltaType, 'total_seconds', no_unliteral=True)
def pd_td_total_seconds(td):

    def impl(td):
        return td._value // 1000 / 10 ** 6
    return impl


def overload_add_operator_datetime_timedelta(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            val = lhs.value + rhs.value
            return pd.Timedelta(val)
        return impl
    if lhs == pd_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            orr__isi = (rhs.microseconds + (rhs.seconds + rhs.days * 60 * 
                60 * 24) * 1000 * 1000) * 1000
            val = lhs.value + orr__isi
            return pd.Timedelta(val)
        return impl
    if lhs == datetime_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            uuw__wrkd = (lhs.microseconds + (lhs.seconds + lhs.days * 60 * 
                60 * 24) * 1000 * 1000) * 1000
            val = uuw__wrkd + rhs.value
            return pd.Timedelta(val)
        return impl
    if lhs == pd_timedelta_type and rhs == datetime_datetime_type:
        from bodo.hiframes.pd_timestamp_ext import compute_pd_timestamp

        def impl(lhs, rhs):
            xtb__wtu = rhs.toordinal()
            fgl__kej = rhs.second + rhs.minute * 60 + rhs.hour * 3600
            ylp__qhhl = rhs.microsecond
            opgf__uosf = lhs.value // 1000
            zyiyj__pascd = lhs.nanoseconds
            tbtok__wis = ylp__qhhl + opgf__uosf
            apkq__rcnvf = 1000000 * (xtb__wtu * 86400 + fgl__kej) + tbtok__wis
            gqh__jgse = zyiyj__pascd
            return compute_pd_timestamp(apkq__rcnvf, gqh__jgse)
        return impl
    if lhs == datetime_datetime_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return lhs + rhs.to_pytimedelta()
        return impl
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            d = lhs.days + rhs.days
            s = lhs.seconds + rhs.seconds
            us = lhs.microseconds + rhs.microseconds
            return datetime.timedelta(d, s, us)
        return impl
    if lhs == datetime_timedelta_type and rhs == datetime_datetime_type:

        def impl(lhs, rhs):
            fqk__glvg = datetime.timedelta(rhs.toordinal(), hours=rhs.hour,
                minutes=rhs.minute, seconds=rhs.second, microseconds=rhs.
                microsecond)
            fqk__glvg = fqk__glvg + lhs
            dpkmk__ftte, baqo__ciuz = divmod(fqk__glvg.seconds, 3600)
            wdh__whl, ndu__aiqj = divmod(baqo__ciuz, 60)
            if 0 < fqk__glvg.days <= _MAXORDINAL:
                d = bodo.hiframes.datetime_date_ext.fromordinal_impl(fqk__glvg
                    .days)
                return datetime.datetime(d.year, d.month, d.day,
                    dpkmk__ftte, wdh__whl, ndu__aiqj, fqk__glvg.microseconds)
            raise OverflowError('result out of range')
        return impl
    if lhs == datetime_datetime_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            fqk__glvg = datetime.timedelta(lhs.toordinal(), hours=lhs.hour,
                minutes=lhs.minute, seconds=lhs.second, microseconds=lhs.
                microsecond)
            fqk__glvg = fqk__glvg + rhs
            dpkmk__ftte, baqo__ciuz = divmod(fqk__glvg.seconds, 3600)
            wdh__whl, ndu__aiqj = divmod(baqo__ciuz, 60)
            if 0 < fqk__glvg.days <= _MAXORDINAL:
                d = bodo.hiframes.datetime_date_ext.fromordinal_impl(fqk__glvg
                    .days)
                return datetime.datetime(d.year, d.month, d.day,
                    dpkmk__ftte, wdh__whl, ndu__aiqj, fqk__glvg.microseconds)
            raise OverflowError('result out of range')
        return impl


def overload_sub_operator_datetime_timedelta(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            zhs__tvcs = lhs.value - rhs.value
            return pd.Timedelta(zhs__tvcs)
        return impl
    if lhs == pd_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            return lhs + -rhs
        return impl
    if lhs == datetime_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return lhs + -rhs
        return impl
    if lhs == datetime_datetime_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return lhs + -rhs
        return impl
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            d = lhs.days - rhs.days
            s = lhs.seconds - rhs.seconds
            us = lhs.microseconds - rhs.microseconds
            return datetime.timedelta(d, s, us)
        return impl
    if lhs == datetime_datetime_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            return lhs + -rhs
        return impl
    if lhs == datetime_timedelta_array_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            irj__cnk = lhs
            numba.parfors.parfor.init_prange()
            n = len(irj__cnk)
            A = alloc_datetime_timedelta_array(n)
            for rdx__arm in numba.parfors.parfor.internal_prange(n):
                A[rdx__arm] = irj__cnk[rdx__arm] - rhs
            return A
        return impl


def overload_mul_operator_timedelta(lhs, rhs):
    if lhs == pd_timedelta_type and isinstance(rhs, types.Integer):

        def impl(lhs, rhs):
            return pd.Timedelta(lhs.value * rhs)
        return impl
    elif isinstance(lhs, types.Integer) and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return pd.Timedelta(rhs.value * lhs)
        return impl
    if lhs == datetime_timedelta_type and isinstance(rhs, types.Integer):

        def impl(lhs, rhs):
            d = lhs.days * rhs
            s = lhs.seconds * rhs
            us = lhs.microseconds * rhs
            return datetime.timedelta(d, s, us)
        return impl
    elif isinstance(lhs, types.Integer) and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            d = lhs * rhs.days
            s = lhs * rhs.seconds
            us = lhs * rhs.microseconds
            return datetime.timedelta(d, s, us)
        return impl


def overload_floordiv_operator_pd_timedelta(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return lhs.value // rhs.value
        return impl
    elif lhs == pd_timedelta_type and isinstance(rhs, types.Integer):

        def impl(lhs, rhs):
            return pd.Timedelta(lhs.value // rhs)
        return impl


def overload_truediv_operator_pd_timedelta(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return lhs.value / rhs.value
        return impl
    elif lhs == pd_timedelta_type and isinstance(rhs, types.Integer):

        def impl(lhs, rhs):
            return pd.Timedelta(int(lhs.value / rhs))
        return impl


def overload_mod_operator_timedeltas(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            return pd.Timedelta(lhs.value % rhs.value)
        return impl
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            htxny__xkeql = _to_microseconds(lhs) % _to_microseconds(rhs)
            return datetime.timedelta(0, 0, htxny__xkeql)
        return impl


def pd_create_cmp_op_overload(op):

    def overload_pd_timedelta_cmp(lhs, rhs):
        if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

            def impl(lhs, rhs):
                return op(lhs.value, rhs.value)
            return impl
        if lhs == pd_timedelta_type and rhs == bodo.timedelta64ns:
            return lambda lhs, rhs: op(bodo.hiframes.pd_timestamp_ext.
                integer_to_timedelta64(lhs.value), rhs)
        if lhs == bodo.timedelta64ns and rhs == pd_timedelta_type:
            return lambda lhs, rhs: op(lhs, bodo.hiframes.pd_timestamp_ext.
                integer_to_timedelta64(rhs.value))
    return overload_pd_timedelta_cmp


@overload(operator.neg, no_unliteral=True)
def pd_timedelta_neg(lhs):
    if lhs == pd_timedelta_type:

        def impl(lhs):
            return pd.Timedelta(-lhs.value)
        return impl


@overload(operator.pos, no_unliteral=True)
def pd_timedelta_pos(lhs):
    if lhs == pd_timedelta_type:

        def impl(lhs):
            return lhs
        return impl


@overload(divmod, no_unliteral=True)
def pd_timedelta_divmod(lhs, rhs):
    if lhs == pd_timedelta_type and rhs == pd_timedelta_type:

        def impl(lhs, rhs):
            bacm__gqdo, htxny__xkeql = divmod(lhs.value, rhs.value)
            return bacm__gqdo, pd.Timedelta(htxny__xkeql)
        return impl


@overload(abs, no_unliteral=True)
def pd_timedelta_abs(lhs):
    if lhs == pd_timedelta_type:

        def impl(lhs):
            if lhs.value < 0:
                return -lhs
            else:
                return lhs
        return impl


class DatetimeTimeDeltaType(types.Type):

    def __init__(self):
        super(DatetimeTimeDeltaType, self).__init__(name=
            'DatetimeTimeDeltaType()')


datetime_timedelta_type = DatetimeTimeDeltaType()


@typeof_impl.register(datetime.timedelta)
def typeof_datetime_timedelta(val, c):
    return datetime_timedelta_type


@register_model(DatetimeTimeDeltaType)
class DatetimeTimeDeltaModel(models.StructModel):

    def __init__(self, dmm, fe_type):
        fygwz__yaxmg = [('days', types.int64), ('seconds', types.int64), (
            'microseconds', types.int64)]
        super(DatetimeTimeDeltaModel, self).__init__(dmm, fe_type, fygwz__yaxmg
            )


@box(DatetimeTimeDeltaType)
def box_datetime_timedelta(typ, val, c):
    qolvn__gslyn = cgutils.create_struct_proxy(typ)(c.context, c.builder,
        value=val)
    ohxzk__iasz = c.pyapi.long_from_longlong(qolvn__gslyn.days)
    ageew__womnz = c.pyapi.long_from_longlong(qolvn__gslyn.seconds)
    pghjv__lzq = c.pyapi.long_from_longlong(qolvn__gslyn.microseconds)
    xwg__ipx = c.pyapi.unserialize(c.pyapi.serialize_object(datetime.timedelta)
        )
    res = c.pyapi.call_function_objargs(xwg__ipx, (ohxzk__iasz,
        ageew__womnz, pghjv__lzq))
    c.pyapi.decref(ohxzk__iasz)
    c.pyapi.decref(ageew__womnz)
    c.pyapi.decref(pghjv__lzq)
    c.pyapi.decref(xwg__ipx)
    return res


@unbox(DatetimeTimeDeltaType)
def unbox_datetime_timedelta(typ, val, c):
    ohxzk__iasz = c.pyapi.object_getattr_string(val, 'days')
    ageew__womnz = c.pyapi.object_getattr_string(val, 'seconds')
    pghjv__lzq = c.pyapi.object_getattr_string(val, 'microseconds')
    jiwd__ebig = c.pyapi.long_as_longlong(ohxzk__iasz)
    ifg__wqip = c.pyapi.long_as_longlong(ageew__womnz)
    mkisw__jsz = c.pyapi.long_as_longlong(pghjv__lzq)
    qolvn__gslyn = cgutils.create_struct_proxy(typ)(c.context, c.builder)
    qolvn__gslyn.days = jiwd__ebig
    qolvn__gslyn.seconds = ifg__wqip
    qolvn__gslyn.microseconds = mkisw__jsz
    c.pyapi.decref(ohxzk__iasz)
    c.pyapi.decref(ageew__womnz)
    c.pyapi.decref(pghjv__lzq)
    dsw__fzzq = cgutils.is_not_null(c.builder, c.pyapi.err_occurred())
    return NativeValue(qolvn__gslyn._getvalue(), is_error=dsw__fzzq)


@lower_constant(DatetimeTimeDeltaType)
def lower_constant_datetime_timedelta(context, builder, ty, pyval):
    days = context.get_constant(types.int64, pyval.days)
    seconds = context.get_constant(types.int64, pyval.seconds)
    microseconds = context.get_constant(types.int64, pyval.microseconds)
    return lir.Constant.literal_struct([days, seconds, microseconds])


@overload(datetime.timedelta, no_unliteral=True)
def datetime_timedelta(days=0, seconds=0, microseconds=0, milliseconds=0,
    minutes=0, hours=0, weeks=0):

    def impl_timedelta(days=0, seconds=0, microseconds=0, milliseconds=0,
        minutes=0, hours=0, weeks=0):
        d = s = us = 0
        days += weeks * 7
        seconds += minutes * 60 + hours * 3600
        microseconds += milliseconds * 1000
        d = days
        days, seconds = divmod(seconds, 24 * 3600)
        d += days
        s += int(seconds)
        seconds, us = divmod(microseconds, 1000000)
        days, seconds = divmod(seconds, 24 * 3600)
        d += days
        s += seconds
        return init_timedelta(d, s, us)
    return impl_timedelta


@intrinsic
def init_timedelta(typingctx, d, s, us):

    def codegen(context, builder, signature, args):
        typ = signature.return_type
        timedelta = cgutils.create_struct_proxy(typ)(context, builder)
        timedelta.days = args[0]
        timedelta.seconds = args[1]
        timedelta.microseconds = args[2]
        return timedelta._getvalue()
    return DatetimeTimeDeltaType()(d, s, us), codegen


make_attribute_wrapper(DatetimeTimeDeltaType, 'days', '_days')
make_attribute_wrapper(DatetimeTimeDeltaType, 'seconds', '_seconds')
make_attribute_wrapper(DatetimeTimeDeltaType, 'microseconds', '_microseconds')


@overload_attribute(DatetimeTimeDeltaType, 'days')
def timedelta_get_days(td):

    def impl(td):
        return td._days
    return impl


@overload_attribute(DatetimeTimeDeltaType, 'seconds')
def timedelta_get_seconds(td):

    def impl(td):
        return td._seconds
    return impl


@overload_attribute(DatetimeTimeDeltaType, 'microseconds')
def timedelta_get_microseconds(td):

    def impl(td):
        return td._microseconds
    return impl


@overload_method(DatetimeTimeDeltaType, 'total_seconds', no_unliteral=True)
def total_seconds(td):

    def impl(td):
        return ((td._days * 86400 + td._seconds) * 10 ** 6 + td._microseconds
            ) / 10 ** 6
    return impl


@overload_method(DatetimeTimeDeltaType, '__hash__', no_unliteral=True)
def __hash__(td):

    def impl(td):
        return hash((td._days, td._seconds, td._microseconds))
    return impl


@register_jitable
def _to_nanoseconds(td):
    return np.int64(((td._days * 86400 + td._seconds) * 1000000 + td.
        _microseconds) * 1000)


@register_jitable
def _to_microseconds(td):
    return (td._days * (24 * 3600) + td._seconds) * 1000000 + td._microseconds


@register_jitable
def _cmp(x, y):
    return 0 if x == y else 1 if x > y else -1


@register_jitable
def _getstate(td):
    return td._days, td._seconds, td._microseconds


@register_jitable
def _divide_and_round(a, b):
    bacm__gqdo, htxny__xkeql = divmod(a, b)
    htxny__xkeql *= 2
    yomvw__kulx = htxny__xkeql > b if b > 0 else htxny__xkeql < b
    if yomvw__kulx or htxny__xkeql == b and bacm__gqdo % 2 == 1:
        bacm__gqdo += 1
    return bacm__gqdo


_MAXORDINAL = 3652059


def overload_floordiv_operator_dt_timedelta(lhs, rhs):
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            us = _to_microseconds(lhs)
            return us // _to_microseconds(rhs)
        return impl
    elif lhs == datetime_timedelta_type and rhs == types.int64:

        def impl(lhs, rhs):
            us = _to_microseconds(lhs)
            return datetime.timedelta(0, 0, us // rhs)
        return impl


def overload_truediv_operator_dt_timedelta(lhs, rhs):
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            us = _to_microseconds(lhs)
            return us / _to_microseconds(rhs)
        return impl
    elif lhs == datetime_timedelta_type and rhs == types.int64:

        def impl(lhs, rhs):
            us = _to_microseconds(lhs)
            return datetime.timedelta(0, 0, _divide_and_round(us, rhs))
        return impl


def create_cmp_op_overload(op):

    def overload_timedelta_cmp(lhs, rhs):
        if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

            def impl(lhs, rhs):
                vvdgy__qwbrn = _cmp(_getstate(lhs), _getstate(rhs))
                return op(vvdgy__qwbrn, 0)
            return impl
    return overload_timedelta_cmp


@overload(operator.neg, no_unliteral=True)
def timedelta_neg(lhs):
    if lhs == datetime_timedelta_type:

        def impl(lhs):
            return datetime.timedelta(-lhs.days, -lhs.seconds, -lhs.
                microseconds)
        return impl


@overload(operator.pos, no_unliteral=True)
def timedelta_pos(lhs):
    if lhs == datetime_timedelta_type:

        def impl(lhs):
            return lhs
        return impl


@overload(divmod, no_unliteral=True)
def timedelta_divmod(lhs, rhs):
    if lhs == datetime_timedelta_type and rhs == datetime_timedelta_type:

        def impl(lhs, rhs):
            bacm__gqdo, htxny__xkeql = divmod(_to_microseconds(lhs),
                _to_microseconds(rhs))
            return bacm__gqdo, datetime.timedelta(0, 0, htxny__xkeql)
        return impl


@overload(abs, no_unliteral=True)
def timedelta_abs(lhs):
    if lhs == datetime_timedelta_type:

        def impl(lhs):
            if lhs.days < 0:
                return -lhs
            else:
                return lhs
        return impl


@intrinsic
def cast_numpy_timedelta_to_int(typingctx, val=None):
    assert val in (types.NPTimedelta('ns'), types.int64)

    def codegen(context, builder, signature, args):
        return args[0]
    return types.int64(val), codegen


@overload(bool, no_unliteral=True)
def timedelta_to_bool(timedelta):
    if timedelta != datetime_timedelta_type:
        return
    jrke__zbeg = datetime.timedelta(0)

    def impl(timedelta):
        return timedelta != jrke__zbeg
    return impl


class DatetimeTimeDeltaArrayType(types.ArrayCompatible):

    def __init__(self):
        super(DatetimeTimeDeltaArrayType, self).__init__(name=
            'DatetimeTimeDeltaArrayType()')

    @property
    def as_array(self):
        return types.Array(types.undefined, 1, 'C')

    @property
    def dtype(self):
        return datetime_timedelta_type

    def copy(self):
        return DatetimeTimeDeltaArrayType()


datetime_timedelta_array_type = DatetimeTimeDeltaArrayType()
types.datetime_timedelta_array_type = datetime_timedelta_array_type
days_data_type = types.Array(types.int64, 1, 'C')
seconds_data_type = types.Array(types.int64, 1, 'C')
microseconds_data_type = types.Array(types.int64, 1, 'C')
nulls_type = types.Array(types.uint8, 1, 'C')


@register_model(DatetimeTimeDeltaArrayType)
class DatetimeTimeDeltaArrayModel(models.StructModel):

    def __init__(self, dmm, fe_type):
        fygwz__yaxmg = [('days_data', days_data_type), ('seconds_data',
            seconds_data_type), ('microseconds_data',
            microseconds_data_type), ('null_bitmap', nulls_type)]
        models.StructModel.__init__(self, dmm, fe_type, fygwz__yaxmg)


make_attribute_wrapper(DatetimeTimeDeltaArrayType, 'days_data', '_days_data')
make_attribute_wrapper(DatetimeTimeDeltaArrayType, 'seconds_data',
    '_seconds_data')
make_attribute_wrapper(DatetimeTimeDeltaArrayType, 'microseconds_data',
    '_microseconds_data')
make_attribute_wrapper(DatetimeTimeDeltaArrayType, 'null_bitmap',
    '_null_bitmap')


@overload_method(DatetimeTimeDeltaArrayType, 'copy', no_unliteral=True)
def overload_datetime_timedelta_arr_copy(A):
    return (lambda A: bodo.hiframes.datetime_timedelta_ext.
        init_datetime_timedelta_array(A._days_data.copy(), A._seconds_data.
        copy(), A._microseconds_data.copy(), A._null_bitmap.copy()))


@unbox(DatetimeTimeDeltaArrayType)
def unbox_datetime_timedelta_array(typ, val, c):
    n = bodo.utils.utils.object_length(c, val)
    gjva__fwyb = types.Array(types.intp, 1, 'C')
    galfq__nfivr = bodo.utils.utils._empty_nd_impl(c.context, c.builder,
        gjva__fwyb, [n])
    nqw__kpowt = bodo.utils.utils._empty_nd_impl(c.context, c.builder,
        gjva__fwyb, [n])
    rvyy__plg = bodo.utils.utils._empty_nd_impl(c.context, c.builder,
        gjva__fwyb, [n])
    miwpw__ozyo = c.builder.udiv(c.builder.add(n, lir.Constant(lir.IntType(
        64), 7)), lir.Constant(lir.IntType(64), 8))
    nrcr__rqu = bodo.utils.utils._empty_nd_impl(c.context, c.builder, types
        .Array(types.uint8, 1, 'C'), [miwpw__ozyo])
    ezyrh__xam = lir.FunctionType(lir.VoidType(), [lir.IntType(8).
        as_pointer(), lir.IntType(64), lir.IntType(64).as_pointer(), lir.
        IntType(64).as_pointer(), lir.IntType(64).as_pointer(), lir.IntType
        (8).as_pointer()])
    sxzb__ygafy = cgutils.get_or_insert_function(c.builder.module,
        ezyrh__xam, name='unbox_datetime_timedelta_array')
    c.builder.call(sxzb__ygafy, [val, n, galfq__nfivr.data, nqw__kpowt.data,
        rvyy__plg.data, nrcr__rqu.data])
    aljlc__lzrbc = cgutils.create_struct_proxy(typ)(c.context, c.builder)
    aljlc__lzrbc.days_data = galfq__nfivr._getvalue()
    aljlc__lzrbc.seconds_data = nqw__kpowt._getvalue()
    aljlc__lzrbc.microseconds_data = rvyy__plg._getvalue()
    aljlc__lzrbc.null_bitmap = nrcr__rqu._getvalue()
    dsw__fzzq = cgutils.is_not_null(c.builder, c.pyapi.err_occurred())
    return NativeValue(aljlc__lzrbc._getvalue(), is_error=dsw__fzzq)


@box(DatetimeTimeDeltaArrayType)
def box_datetime_timedelta_array(typ, val, c):
    irj__cnk = cgutils.create_struct_proxy(typ)(c.context, c.builder, val)
    galfq__nfivr = c.context.make_array(types.Array(types.int64, 1, 'C'))(c
        .context, c.builder, irj__cnk.days_data)
    nqw__kpowt = c.context.make_array(types.Array(types.int64, 1, 'C'))(c.
        context, c.builder, irj__cnk.seconds_data).data
    rvyy__plg = c.context.make_array(types.Array(types.int64, 1, 'C'))(c.
        context, c.builder, irj__cnk.microseconds_data).data
    ptpv__gbeyn = c.context.make_array(types.Array(types.uint8, 1, 'C'))(c.
        context, c.builder, irj__cnk.null_bitmap).data
    n = c.builder.extract_value(galfq__nfivr.shape, 0)
    ezyrh__xam = lir.FunctionType(c.pyapi.pyobj, [lir.IntType(64), lir.
        IntType(64).as_pointer(), lir.IntType(64).as_pointer(), lir.IntType
        (64).as_pointer(), lir.IntType(8).as_pointer()])
    fxhq__crh = cgutils.get_or_insert_function(c.builder.module, ezyrh__xam,
        name='box_datetime_timedelta_array')
    slfdx__xwd = c.builder.call(fxhq__crh, [n, galfq__nfivr.data,
        nqw__kpowt, rvyy__plg, ptpv__gbeyn])
    c.context.nrt.decref(c.builder, typ, val)
    return slfdx__xwd


@intrinsic
def init_datetime_timedelta_array(typingctx, days_data, seconds_data,
    microseconds_data, nulls=None):
    assert days_data == types.Array(types.int64, 1, 'C')
    assert seconds_data == types.Array(types.int64, 1, 'C')
    assert microseconds_data == types.Array(types.int64, 1, 'C')
    assert nulls == types.Array(types.uint8, 1, 'C')

    def codegen(context, builder, signature, args):
        dsvd__dzpca, vro__fmj, bgm__webh, jul__iotn = args
        nddvr__ruwk = cgutils.create_struct_proxy(signature.return_type)(
            context, builder)
        nddvr__ruwk.days_data = dsvd__dzpca
        nddvr__ruwk.seconds_data = vro__fmj
        nddvr__ruwk.microseconds_data = bgm__webh
        nddvr__ruwk.null_bitmap = jul__iotn
        context.nrt.incref(builder, signature.args[0], dsvd__dzpca)
        context.nrt.incref(builder, signature.args[1], vro__fmj)
        context.nrt.incref(builder, signature.args[2], bgm__webh)
        context.nrt.incref(builder, signature.args[3], jul__iotn)
        return nddvr__ruwk._getvalue()
    doan__waqj = datetime_timedelta_array_type(days_data, seconds_data,
        microseconds_data, nulls)
    return doan__waqj, codegen


@lower_constant(DatetimeTimeDeltaArrayType)
def lower_constant_datetime_timedelta_arr(context, builder, typ, pyval):
    n = len(pyval)
    galfq__nfivr = np.empty(n, np.int64)
    nqw__kpowt = np.empty(n, np.int64)
    rvyy__plg = np.empty(n, np.int64)
    wgf__bgwn = np.empty(n + 7 >> 3, np.uint8)
    for rdx__arm, s in enumerate(pyval):
        kphwb__tbej = pd.isna(s)
        bodo.libs.int_arr_ext.set_bit_to_arr(wgf__bgwn, rdx__arm, int(not
            kphwb__tbej))
        if not kphwb__tbej:
            galfq__nfivr[rdx__arm] = s.days
            nqw__kpowt[rdx__arm] = s.seconds
            rvyy__plg[rdx__arm] = s.microseconds
    glskb__tzqhs = context.get_constant_generic(builder, days_data_type,
        galfq__nfivr)
    tfazj__tzsg = context.get_constant_generic(builder, seconds_data_type,
        nqw__kpowt)
    bauud__dktre = context.get_constant_generic(builder,
        microseconds_data_type, rvyy__plg)
    jxy__sts = context.get_constant_generic(builder, nulls_type, wgf__bgwn)
    return lir.Constant.literal_struct([glskb__tzqhs, tfazj__tzsg,
        bauud__dktre, jxy__sts])


@numba.njit(no_cpython_wrapper=True)
def alloc_datetime_timedelta_array(n):
    galfq__nfivr = np.empty(n, dtype=np.int64)
    nqw__kpowt = np.empty(n, dtype=np.int64)
    rvyy__plg = np.empty(n, dtype=np.int64)
    nulls = np.full(n + 7 >> 3, 255, np.uint8)
    return init_datetime_timedelta_array(galfq__nfivr, nqw__kpowt,
        rvyy__plg, nulls)


def alloc_datetime_timedelta_array_equiv(self, scope, equiv_set, loc, args, kws
    ):
    assert len(args) == 1 and not kws
    return ArrayAnalysis.AnalyzeResult(shape=args[0], pre=[])


(ArrayAnalysis.
    _analyze_op_call_bodo_hiframes_datetime_timedelta_ext_alloc_datetime_timedelta_array
    ) = alloc_datetime_timedelta_array_equiv


@overload(operator.getitem, no_unliteral=True)
def dt_timedelta_arr_getitem(A, ind):
    if A != datetime_timedelta_array_type:
        return
    if isinstance(ind, types.Integer):

        def impl_int(A, ind):
            return datetime.timedelta(days=A._days_data[ind], seconds=A.
                _seconds_data[ind], microseconds=A._microseconds_data[ind])
        return impl_int
    if is_list_like_index_type(ind) and ind.dtype == types.bool_:

        def impl_bool(A, ind):
            ghzq__klop = bodo.utils.conversion.coerce_to_ndarray(ind)
            jyute__dcj = A._null_bitmap
            qaar__gztpa = A._days_data[ghzq__klop]
            mufy__ykpg = A._seconds_data[ghzq__klop]
            mnpz__rzo = A._microseconds_data[ghzq__klop]
            n = len(qaar__gztpa)
            hdaoa__qjh = get_new_null_mask_bool_index(jyute__dcj, ind, n)
            return init_datetime_timedelta_array(qaar__gztpa, mufy__ykpg,
                mnpz__rzo, hdaoa__qjh)
        return impl_bool
    if is_list_like_index_type(ind) and isinstance(ind.dtype, types.Integer):

        def impl(A, ind):
            ghzq__klop = bodo.utils.conversion.coerce_to_ndarray(ind)
            jyute__dcj = A._null_bitmap
            qaar__gztpa = A._days_data[ghzq__klop]
            mufy__ykpg = A._seconds_data[ghzq__klop]
            mnpz__rzo = A._microseconds_data[ghzq__klop]
            n = len(qaar__gztpa)
            hdaoa__qjh = get_new_null_mask_int_index(jyute__dcj, ghzq__klop, n)
            return init_datetime_timedelta_array(qaar__gztpa, mufy__ykpg,
                mnpz__rzo, hdaoa__qjh)
        return impl
    if isinstance(ind, types.SliceType):

        def impl_slice(A, ind):
            n = len(A._days_data)
            jyute__dcj = A._null_bitmap
            qaar__gztpa = np.ascontiguousarray(A._days_data[ind])
            mufy__ykpg = np.ascontiguousarray(A._seconds_data[ind])
            mnpz__rzo = np.ascontiguousarray(A._microseconds_data[ind])
            hdaoa__qjh = get_new_null_mask_slice_index(jyute__dcj, ind, n)
            return init_datetime_timedelta_array(qaar__gztpa, mufy__ykpg,
                mnpz__rzo, hdaoa__qjh)
        return impl_slice
    raise BodoError(
        f'getitem for DatetimeTimedeltaArray with indexing type {ind} not supported.'
        )


@overload(operator.setitem, no_unliteral=True)
def dt_timedelta_arr_setitem(A, ind, val):
    if A != datetime_timedelta_array_type:
        return
    if val == types.none or isinstance(val, types.optional):
        return
    msc__jckc = (
        f"setitem for DatetimeTimedeltaArray with indexing type {ind} received an incorrect 'value' type {val}."
        )
    if isinstance(ind, types.Integer):
        if types.unliteral(val) == datetime_timedelta_type:

            def impl(A, ind, val):
                A._days_data[ind] = val._days
                A._seconds_data[ind] = val._seconds
                A._microseconds_data[ind] = val._microseconds
                bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap, ind, 1)
            return impl
        else:
            raise BodoError(msc__jckc)
    if not (is_iterable_type(val) and val.dtype == bodo.
        datetime_timedelta_type or types.unliteral(val) ==
        datetime_timedelta_type):
        raise BodoError(msc__jckc)
    if is_list_like_index_type(ind) and isinstance(ind.dtype, types.Integer):
        if types.unliteral(val) == datetime_timedelta_type:

            def impl_arr_ind_scalar(A, ind, val):
                n = len(A)
                for rdx__arm in range(n):
                    A._days_data[ind[rdx__arm]] = val._days
                    A._seconds_data[ind[rdx__arm]] = val._seconds
                    A._microseconds_data[ind[rdx__arm]] = val._microseconds
                    bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap,
                        ind[rdx__arm], 1)
            return impl_arr_ind_scalar
        else:

            def impl_arr_ind(A, ind, val):
                val = bodo.utils.conversion.coerce_to_array(val,
                    use_nullable_array=True)
                n = len(val._days_data)
                for rdx__arm in range(n):
                    A._days_data[ind[rdx__arm]] = val._days_data[rdx__arm]
                    A._seconds_data[ind[rdx__arm]] = val._seconds_data[rdx__arm
                        ]
                    A._microseconds_data[ind[rdx__arm]
                        ] = val._microseconds_data[rdx__arm]
                    oxmw__hltf = bodo.libs.int_arr_ext.get_bit_bitmap_arr(val
                        ._null_bitmap, rdx__arm)
                    bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap,
                        ind[rdx__arm], oxmw__hltf)
            return impl_arr_ind
    if is_list_like_index_type(ind) and ind.dtype == types.bool_:
        if types.unliteral(val) == datetime_timedelta_type:

            def impl_bool_ind_mask_scalar(A, ind, val):
                n = len(ind)
                for rdx__arm in range(n):
                    if not bodo.libs.array_kernels.isna(ind, rdx__arm) and ind[
                        rdx__arm]:
                        A._days_data[rdx__arm] = val._days
                        A._seconds_data[rdx__arm] = val._seconds
                        A._microseconds_data[rdx__arm] = val._microseconds
                        bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap,
                            rdx__arm, 1)
            return impl_bool_ind_mask_scalar
        else:

            def impl_bool_ind_mask(A, ind, val):
                val = bodo.utils.conversion.coerce_to_array(val,
                    use_nullable_array=True)
                n = len(ind)
                tws__kimf = 0
                for rdx__arm in range(n):
                    if not bodo.libs.array_kernels.isna(ind, rdx__arm) and ind[
                        rdx__arm]:
                        A._days_data[rdx__arm] = val._days_data[tws__kimf]
                        A._seconds_data[rdx__arm] = val._seconds_data[tws__kimf
                            ]
                        A._microseconds_data[rdx__arm
                            ] = val._microseconds_data[tws__kimf]
                        oxmw__hltf = bodo.libs.int_arr_ext.get_bit_bitmap_arr(
                            val._null_bitmap, tws__kimf)
                        bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap,
                            rdx__arm, oxmw__hltf)
                        tws__kimf += 1
            return impl_bool_ind_mask
    if isinstance(ind, types.SliceType):
        if types.unliteral(val) == datetime_timedelta_type:

            def impl_slice_scalar(A, ind, val):
                sidkz__vahlo = numba.cpython.unicode._normalize_slice(ind,
                    len(A))
                for rdx__arm in range(sidkz__vahlo.start, sidkz__vahlo.stop,
                    sidkz__vahlo.step):
                    A._days_data[rdx__arm] = val._days
                    A._seconds_data[rdx__arm] = val._seconds
                    A._microseconds_data[rdx__arm] = val._microseconds
                    bodo.libs.int_arr_ext.set_bit_to_arr(A._null_bitmap,
                        rdx__arm, 1)
            return impl_slice_scalar
        else:

            def impl_slice_mask(A, ind, val):
                val = bodo.utils.conversion.coerce_to_array(val,
                    use_nullable_array=True)
                n = len(A._days_data)
                A._days_data[ind] = val._days_data
                A._seconds_data[ind] = val._seconds_data
                A._microseconds_data[ind] = val._microseconds_data
                ffeag__eec = val._null_bitmap.copy()
                setitem_slice_index_null_bits(A._null_bitmap, ffeag__eec,
                    ind, n)
            return impl_slice_mask
    raise BodoError(
        f'setitem for DatetimeTimedeltaArray with indexing type {ind} not supported.'
        )


@overload(len, no_unliteral=True)
def overload_len_datetime_timedelta_arr(A):
    if A == datetime_timedelta_array_type:
        return lambda A: len(A._days_data)


@overload_attribute(DatetimeTimeDeltaArrayType, 'shape')
def overload_datetime_timedelta_arr_shape(A):
    return lambda A: (len(A._days_data),)


@overload_attribute(DatetimeTimeDeltaArrayType, 'nbytes')
def timedelta_arr_nbytes_overload(A):
    return (lambda A: A._days_data.nbytes + A._seconds_data.nbytes + A.
        _microseconds_data.nbytes + A._null_bitmap.nbytes)


def overload_datetime_timedelta_arr_sub(arg1, arg2):
    if (arg1 == datetime_timedelta_array_type and arg2 ==
        datetime_timedelta_type):

        def impl(arg1, arg2):
            irj__cnk = arg1
            numba.parfors.parfor.init_prange()
            n = len(irj__cnk)
            A = alloc_datetime_timedelta_array(n)
            for rdx__arm in numba.parfors.parfor.internal_prange(n):
                A[rdx__arm] = irj__cnk[rdx__arm] - arg2
            return A
        return impl


def create_cmp_op_overload_arr(op):

    def overload_date_arr_cmp(lhs, rhs):
        if op == operator.ne:
            elvzv__nxamk = True
        else:
            elvzv__nxamk = False
        if (lhs == datetime_timedelta_array_type and rhs ==
            datetime_timedelta_array_type):

            def impl(lhs, rhs):
                numba.parfors.parfor.init_prange()
                n = len(lhs)
                ekt__gaa = bodo.libs.bool_arr_ext.alloc_bool_array(n)
                for rdx__arm in numba.parfors.parfor.internal_prange(n):
                    qwf__gqi = bodo.libs.array_kernels.isna(lhs, rdx__arm)
                    zkdxj__ofao = bodo.libs.array_kernels.isna(rhs, rdx__arm)
                    if qwf__gqi or zkdxj__ofao:
                        iasoy__kkh = elvzv__nxamk
                    else:
                        iasoy__kkh = op(lhs[rdx__arm], rhs[rdx__arm])
                    ekt__gaa[rdx__arm] = iasoy__kkh
                return ekt__gaa
            return impl
        elif lhs == datetime_timedelta_array_type:

            def impl(lhs, rhs):
                numba.parfors.parfor.init_prange()
                n = len(lhs)
                ekt__gaa = bodo.libs.bool_arr_ext.alloc_bool_array(n)
                for rdx__arm in numba.parfors.parfor.internal_prange(n):
                    oxmw__hltf = bodo.libs.array_kernels.isna(lhs, rdx__arm)
                    if oxmw__hltf:
                        iasoy__kkh = elvzv__nxamk
                    else:
                        iasoy__kkh = op(lhs[rdx__arm], rhs)
                    ekt__gaa[rdx__arm] = iasoy__kkh
                return ekt__gaa
            return impl
        elif rhs == datetime_timedelta_array_type:

            def impl(lhs, rhs):
                numba.parfors.parfor.init_prange()
                n = len(rhs)
                ekt__gaa = bodo.libs.bool_arr_ext.alloc_bool_array(n)
                for rdx__arm in numba.parfors.parfor.internal_prange(n):
                    oxmw__hltf = bodo.libs.array_kernels.isna(rhs, rdx__arm)
                    if oxmw__hltf:
                        iasoy__kkh = elvzv__nxamk
                    else:
                        iasoy__kkh = op(lhs, rhs[rdx__arm])
                    ekt__gaa[rdx__arm] = iasoy__kkh
                return ekt__gaa
            return impl
    return overload_date_arr_cmp


timedelta_unsupported_attrs = ['asm8', 'resolution_string', 'freq',
    'is_populated']
timedelta_unsupported_methods = ['isoformat']


def _intstall_pd_timedelta_unsupported():
    from bodo.utils.typing import create_unsupported_overload
    for xqyo__pkf in timedelta_unsupported_attrs:
        iorb__wyd = 'pandas.Timedelta.' + xqyo__pkf
        overload_attribute(PDTimeDeltaType, xqyo__pkf)(
            create_unsupported_overload(iorb__wyd))
    for rks__jlz in timedelta_unsupported_methods:
        iorb__wyd = 'pandas.Timedelta.' + rks__jlz
        overload_method(PDTimeDeltaType, rks__jlz)(create_unsupported_overload
            (iorb__wyd + '()'))


_intstall_pd_timedelta_unsupported()
